import { jsx } from 'react/jsx-runtime';
import React from 'react';
import { EditorState } from '@codemirror/state';
import { EditorView } from '@codemirror/view';

// credit: https://github.com/JakeGinnivan/merge-refs-hook
function setRefs(ref, value) {
    if (typeof ref === 'function') {
        ref(value);
    } else if (ref) {
        ref.current = value;
    }
}
function useMergeRefs(forwardedRef, localRef) {
    return React.useCallback(function(value) {
        setRefs(forwardedRef, value);
        setRefs(localRef, value);
    }, [
        forwardedRef,
        localRef
    ]);
}

function _arrayWithHoles(arr) {
    if (Array.isArray(arr)) return arr;
}
function _arrayWithoutHoles(arr) {
    if (Array.isArray(arr)) {
        for(var i = 0, arr2 = new Array(arr.length); i < arr.length; i++){
            arr2[i] = arr[i];
        }
        return arr2;
    }
}
function _defineProperty(obj, key, value) {
    if (key in obj) {
        Object.defineProperty(obj, key, {
            value: value,
            enumerable: true,
            configurable: true,
            writable: true
        });
    } else {
        obj[key] = value;
    }
    return obj;
}
function _iterableToArray(iter) {
    if (Symbol.iterator in Object(iter) || Object.prototype.toString.call(iter) === "[object Arguments]") return Array.from(iter);
}
function _iterableToArrayLimit(arr, i) {
    var _arr = [];
    var _n = true;
    var _d = false;
    var _e = undefined;
    try {
        for(var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true){
            _arr.push(_s.value);
            if (i && _arr.length === i) break;
        }
    } catch (err) {
        _d = true;
        _e = err;
    } finally{
        try {
            if (!_n && _i["return"] != null) _i["return"]();
        } finally{
            if (_d) throw _e;
        }
    }
    return _arr;
}
function _nonIterableRest() {
    throw new TypeError("Invalid attempt to destructure non-iterable instance");
}
function _nonIterableSpread() {
    throw new TypeError("Invalid attempt to spread non-iterable instance");
}
function _objectSpread(target) {
    for(var i = 1; i < arguments.length; i++){
        var source = arguments[i] != null ? arguments[i] : {
        };
        var ownKeys = Object.keys(source);
        if (typeof Object.getOwnPropertySymbols === "function") {
            ownKeys = ownKeys.concat(Object.getOwnPropertySymbols(source).filter(function(sym) {
                return Object.getOwnPropertyDescriptor(source, sym).enumerable;
            }));
        }
        ownKeys.forEach(function(key) {
            _defineProperty(target, key, source[key]);
        });
    }
    return target;
}
function _slicedToArray(arr, i) {
    return _arrayWithHoles(arr) || _iterableToArrayLimit(arr, i) || _nonIterableRest();
}
function _toConsumableArray(arr) {
    return _arrayWithoutHoles(arr) || _iterableToArray(arr) || _nonIterableSpread();
}
var CodeMirror = /*#__PURE__*/ React.forwardRef(function(param, ref) {
    var value = param.value, selection = param.selection, onEditorViewChange = param.onEditorViewChange, onEditorStateChange = param.onEditorStateChange, onUpdate = param.onUpdate, tmp = param.extensions, passedExtensions = tmp === void 0 ? [] : tmp, elementProps = param.elementProps;
    var innerRef = React.useRef(null);
    var mergedRef = useMergeRefs(ref, innerRef);
    var ref1 = _slicedToArray(React.useState(null), 2), editorView = ref1[0], setEditorView = ref1[1];
    React.useEffect(function() {
        var currentEditor = innerRef.current;
        if (!currentEditor) return;
        var view = new EditorView({
            parent: currentEditor
        });
        setEditorView(view);
        if (onEditorViewChange) onEditorViewChange(view);
        return function() {
            return view.destroy();
        };
    }, [
        innerRef
    ]);
    React.useEffect(function() {
        if (!editorView) return;
        var extensions = [];
        if (onUpdate) extensions.push(EditorView.updateListener.of(onUpdate));
        // keep current state, only change extensions
        var editorState = EditorState.create({
            doc: editorView.state.doc.toString(),
            selection: editorView.state.selection,
            extensions: _toConsumableArray(extensions).concat(_toConsumableArray(passedExtensions))
        });
        editorView.setState(editorState);
        if (onEditorStateChange) onEditorStateChange(editorState);
    }, [
        passedExtensions,
        editorView
    ]);
    React.useEffect(function() {
        if (!editorView) return;
        var transaction = editorView.state.update({
            changes: {
                from: 0,
                to: editorView.state.doc.length,
                insert: value
            },
            selection: editorView.state.selection
        });
        editorView.dispatch(transaction);
    }, [
        value,
        editorView
    ]);
    React.useEffect(function() {
        if (!editorView) return;
        var transaction = editorView.state.update({
            selection: selection
        });
        editorView.dispatch(transaction);
    }, [
        selection,
        editorView
    ]);
    return(/*#__PURE__*/ jsx("div", _objectSpread({
        ref: mergedRef
    }, elementProps)));
});
CodeMirror.displayName = 'CodeMirror';

export { CodeMirror as default };
